@@ -17,7 +17,9 @@
  * Authored by: Marco Trevisan <marco.trevisan@canonical.com>
  */
 
-#include <NuxCore/Logger.h>
+#include <core/atoms.h>
+#include <X11/Xatom.h>
+
 #include "DecorationsPriv.h"
 #include "DecorationsForceQuitDialog.h"
 #include "DecorationsEdgeBorders.h"
@@ -35,7 +37,6 @@ namespace decoration
 {
 namespace
 {
-DECLARE_LOGGER(logger, "unity.decoration.window");
 const std::string MENUS_PANEL_NAME = "WindowLIM";
 }
 
@@ -49,20 +50,17 @@ Window::Impl::Impl(Window* parent, CompWindow* win)
   , monitor_(0)
   , dirty_geo_(true)
   , dirty_frame_(false)
+  , client_decorated_(false)
   , deco_elements_(cu::DecorationElement::NONE)
   , last_mwm_decor_(win_->mwmDecor())
   , last_actions_(win_->actions())
+  , panel_id_(MENUS_PANEL_NAME + std::to_string(win_->id()))
   , cv_(Settings::Instance().em())
 {
   active.changed.connect([this] (bool active) {
     bg_textures_.clear();
     if (top_layout_)
-    {
       top_layout_->focused = active;
-
-      if (!active)
-        UnsetAppMenu();
-    }
     RedrawDecorations();
   });
 
@@ -111,7 +109,8 @@ Window::Impl::~Impl()
 
 void Window::Impl::Update()
 {
-  UpdateElements();
+  UpdateClientDecorationsState();
+  UpdateElements(client_decorated_ ? cu::WindowFilter::CLIENTSIDE_DECORATED : cu::WindowFilter::NONE);
 
   if (deco_elements_ & (cu::DecorationElement::EDGE | cu::DecorationElement::BORDER))
     Decorate();
@@ -159,13 +158,8 @@ void Window::Impl::UnsetExtents()
     win_->setWindowFrameExtents(&empty, &empty);
 }
 
-void Window::Impl::SetupExtents()
+void Window::Impl::ComputeBorderExtent(CompWindowExtents& border)
 {
-  if (win_->hasUnmapReference())
-    return;
-
-  CompWindowExtents border;
-
   if (deco_elements_ & cu::DecorationElement::BORDER)
   {
     auto const& sb = Style::Get()->Border();
@@ -174,6 +168,15 @@ void Window::Impl::SetupExtents()
     border.top = cv_->CP(sb.top);
     border.bottom = cv_->CP(sb.bottom);
   }
+}
+
+void Window::Impl::SetupExtents()
+{
+  if (win_->hasUnmapReference())
+    return;
+
+  CompWindowExtents border;
+  ComputeBorderExtent(border);
 
   CompWindowExtents input(border);
 
@@ -193,8 +196,19 @@ void Window::Impl::SetupExtents()
 void Window::Impl::SendFrameExtents()
 {
   UpdateElements(cu::WindowFilter::UNMAPPED);
-  SetupExtents();
-  win_->setWindowFrameExtents(&win_->border(), &win_->input());
+
+  CompWindowExtents border;
+  ComputeBorderExtent(border);
+
+  std::vector<unsigned long> extents(4);
+  extents.push_back(border.left);
+  extents.push_back(border.right);
+  extents.push_back(border.top);
+  extents.push_back(border.bottom);
+
+  XChangeProperty(screen->dpy(), win_->id(), Atoms::frameExtents, XA_CARDINAL, 32,
+                  PropModeReplace, reinterpret_cast<unsigned char *>(extents.data()),
+                  extents.size());
 }
 
 void Window::Impl::UnsetFrame()
@@ -218,10 +232,10 @@ void Window::Impl::UpdateFrame()
   if (win_->shaded())
     frame_geo.height = input.top + input.bottom;
 
-  if (!frame_)
+  if (!frame_ && win_->frame())
     CreateFrame(frame_geo);
 
-  if (frame_geo_ != frame_geo)
+  if (frame_ && frame_geo_ != frame_geo)
     UpdateFrameGeo(frame_geo);
 }
 
@@ -437,7 +451,7 @@ bool Window::Impl::IsMaximized() const
   return (win_->state() & MAXIMIZE_STATE) == MAXIMIZE_STATE;
 }
 
-void Window::Impl::UpdateElements(cu::WindowFilter::Value wf)
+void Window::Impl::UpdateElements(cu::WindowFilter wf)
 {
   if (!parent_->scaled() && IsMaximized())
   {
@@ -448,6 +462,31 @@ void Window::Impl::UpdateElements(cu::WindowFilter::Value wf)
   deco_elements_ = cu::WindowDecorationElements(win_, wf);
 }
 
+void Window::Impl::UpdateClientDecorationsState()
+{
+  if (win_->alpha())
+  {
+    auto const& corners = WindowManager::Default().GetCardinalProperty(win_->id(), atom::_UNITY_GTK_BORDER_RADIUS);
+
+    if (!corners.empty())
+    {
+      enum Corner { TOP_LEFT, TOP_RIGHT, BOTTOM_LEFT, BOTTOM_RIGHT };
+      client_borders_.top = std::max(corners[TOP_LEFT], corners[TOP_RIGHT]);
+      client_borders_.left = std::max(corners[TOP_LEFT], corners[BOTTOM_LEFT]);
+      client_borders_.right = std::max(corners[TOP_RIGHT], corners[BOTTOM_RIGHT]);
+      client_borders_.bottom = std::max(corners[BOTTOM_LEFT], corners[BOTTOM_RIGHT]);
+      client_decorated_ = true;
+      return;
+    }
+  }
+
+  if (client_decorated_)
+  {
+    client_borders_ = CompWindowExtents();
+    client_decorated_ = false;
+  }
+}
+
 bool Window::Impl::ShadowDecorated() const
 {
   return deco_elements_ & cu::DecorationElement::SHADOW;
@@ -619,7 +658,14 @@ void Window::Impl::ComputeShadowQuads()
 
   if (shadows_rect != last_shadow_rect_)
   {
-    auto const& win_region = win_->region();
+    auto win_region = win_->region();
+
+    if (client_decorated_)
+    {
+      win_region.shrink(client_borders_.left + client_borders_.right, client_borders_.top + client_borders_.bottom);
+      win_region.translate(client_borders_.left - client_borders_.right, client_borders_.top - client_borders_.bottom);
+    }
+
     quads[Quads::Pos::TOP_LEFT].region = CompRegion(quads[Quads::Pos::TOP_LEFT].box) - win_region;
     quads[Quads::Pos::TOP_RIGHT].region = CompRegion(quads[Quads::Pos::TOP_RIGHT].box) - win_region;
     quads[Quads::Pos::BOTTOM_LEFT].region = CompRegion(quads[Quads::Pos::BOTTOM_LEFT].box) - win_region;
@@ -663,6 +709,9 @@ void Window::Impl::Draw(GLMatrix const& transformation,
   auto const& clip_region = (mask & PAINT_WINDOW_TRANSFORMED_MASK) ? infiniteRegion : region;
   mask |= PAINT_WINDOW_BLEND_MASK;
 
+  if (win_->alpha() || attrib.opacity != OPAQUE)
+    mask |= PAINT_WINDOW_TRANSLUCENT_MASK;
+
   glwin_->vertexBuffer()->begin();
 
   for (unsigned i = 0; i < shadow_quads_.size(); ++i)
@@ -703,7 +752,7 @@ void Window::Impl::RedrawDecorations()
 
 void Window::Impl::SetupAppMenu()
 {
-  if (!active() || !top_layout_)
+  if (!top_layout_)
     return;
 
   auto const& menu_manager = manager_->impl_->menu_manager_;
@@ -718,8 +767,11 @@ void Window::Impl::SetupAppMenu()
 
   auto menus = std::make_shared<MenuLayout>(menu_manager, win_);
   menus->Setup();
-  menus_ = menus;
 
+  if (menus->Items().empty())
+    return;
+
+  menus_ = menus;
   auto const& grab_edge = grab_edge_.lock();
   sliding_layout->SetInputItem(menus);
   sliding_layout->fadein = menu_manager->fadein();
@@ -761,13 +813,18 @@ void Window::Impl::UpdateAppMenuVisibility()
     sliding_layout->mouse_owner = grab_edge_->mouse_owner();
 }
 
+inline std::string const& Window::Impl::GetMenusPanelID() const
+{
+  return panel_id_;
+}
+
 void Window::Impl::UnsetAppMenu()
 {
   if (!menus_)
     return;
 
   auto const& indicators = manager_->impl_->menu_manager_->Indicators();
-  indicators->SyncGeometries(MENUS_PANEL_NAME, indicator::EntryLocationMap());
+  indicators->SyncGeometries(GetMenusPanelID(), indicator::EntryLocationMap());
   sliding_layout_->SetInputItem(nullptr);
   grab_mouse_changed_->disconnect();
 }
@@ -780,7 +837,7 @@ void Window::Impl::SyncMenusGeometries() const
   auto const& indicators = manager_->impl_->menu_manager_->Indicators();
   indicator::EntryLocationMap map;
   menus_->ChildrenGeometries(map);
-  indicators->SyncGeometries(MENUS_PANEL_NAME, map);
+  indicators->SyncGeometries(GetMenusPanelID(), map);
 }
 
 bool Window::Impl::ActivateMenu(std::string const& entry_id)
