@@ -138,6 +138,7 @@ GtkWidget* sheet_style_window_new(ForceQuitDialog* main_dialog, Window parent_xi
 
   Atom WM_PID = gdk_x11_get_xatom_by_name("_NET_WM_PID");
   Atom WM_CLIENT_MACHINE = gdk_x11_get_xatom_by_name("WM_CLIENT_MACHINE");
+  Atom WM_CLIENT_LEADER = gdk_x11_get_xatom_by_name("WM_CLIENT_LEADER");
 
   gdk_error_trap_push();
   auto& wm = WindowManager::Default();
@@ -168,7 +169,14 @@ GtkWidget* sheet_style_window_new(ForceQuitDialog* main_dialog, Window parent_xi
   auto* screen = gtk_window_get_screen(self);
   gtk_widget_set_visual(GTK_WIDGET(self), gdk_screen_get_rgba_visual(screen));
   gtk_widget_realize(GTK_WIDGET(self));
-  gtk_widget_override_background_color(GTK_WIDGET(self), GTK_STATE_FLAG_NORMAL, nullptr);
+
+  glib::Object<GtkCssProvider> style(gtk_css_provider_new());
+  gtk_css_provider_load_from_data(style, R"(
+    * { background-color: transparent; }
+  )", -1, nullptr);
+
+  auto* style_ctx = gtk_widget_get_style_context(GTK_WIDGET(self));
+  gtk_style_context_add_provider(style_ctx, glib::object_cast<GtkStyleProvider>(style), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
   gtk_container_add(GTK_CONTAINER(self), sheet_style_dialog_new(main_dialog, parent_xid, parent_pid));
 
@@ -189,6 +197,8 @@ GtkWidget* sheet_style_window_new(ForceQuitDialog* main_dialog, Window parent_xi
                   (unsigned char *) parent_hostname.c_str(), parent_hostname.size());
   XChangeProperty(dpy, xid, WM_PID, XA_CARDINAL, 32, PropModeReplace,
                   (unsigned char *) &parent_pid, 1);
+  XChangeProperty(dpy, xid, WM_CLIENT_LEADER, XA_WINDOW, 32, PropModeReplace,
+                  (unsigned char *) &parent_xid, 1);
   XSync(dpy, False);
 
   return GTK_WIDGET(self);
@@ -306,14 +316,25 @@ GtkWidget* sheet_style_dialog_new(ForceQuitDialog* main_dialog, Window parent_xi
 
   auto* content_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
   gtk_container_set_border_width(GTK_CONTAINER(content_box), 10);
+#if GTK_CHECK_VERSION(3, 11, 0)
+  gtk_widget_set_margin_start(content_box, 20);
+  gtk_widget_set_margin_end(content_box, 20);
+#else
   gtk_widget_set_margin_left(content_box, 20);
   gtk_widget_set_margin_right(content_box, 20);
+#endif
   gtk_container_add(GTK_CONTAINER(main_box), content_box);
 
   auto* title = gtk_label_new(_("This window is not responding"));
-  auto* font_desc = pango_font_description_from_string("Ubuntu 17");
-  gtk_widget_override_font(title, font_desc);
-  pango_font_description_free(font_desc);
+  glib::Object<GtkCssProvider> title_style(gtk_css_provider_new());
+  gtk_css_provider_load_from_data(title_style, (R"(
+  GtkLabel {
+    font-size: 17px;
+  })"), -1, nullptr);
+  style_ctx = gtk_widget_get_style_context(title);
+  gtk_style_context_add_provider(style_ctx, glib::object_cast<GtkStyleProvider>(title_style), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  gtk_style_context_add_class(style_ctx, "unity-force-quit");
+
   gtk_widget_set_halign(title, GTK_ALIGN_START);
   gtk_box_pack_start(GTK_BOX(content_box), title, FALSE, FALSE, 0);
 
@@ -325,6 +346,9 @@ GtkWidget* sheet_style_dialog_new(ForceQuitDialog* main_dialog, Window parent_xi
   gtk_box_set_spacing(GTK_BOX(buttons_box), 15);
   gtk_container_set_border_width(GTK_CONTAINER(buttons_box), 5);
   gtk_button_box_set_layout(GTK_BUTTON_BOX(buttons_box), GTK_BUTTONBOX_END);
+  gtk_widget_set_halign(GTK_WIDGET(buttons_box), GTK_ALIGN_END);
+  gtk_widget_set_valign(GTK_WIDGET(buttons_box), GTK_ALIGN_END);
+  gtk_widget_set_margin_top(GTK_WIDGET(buttons_box), 20);
 
   auto* wait_button = gtk_button_new_with_mnemonic(_("_Wait"));
   gtk_container_add(GTK_CONTAINER(buttons_box), wait_button);
@@ -339,11 +363,7 @@ GtkWidget* sheet_style_dialog_new(ForceQuitDialog* main_dialog, Window parent_xi
                         kill_data, [] (gpointer data, GClosure*) { g_free(data); },
                         static_cast<GConnectFlags>(0));
 
-  auto* buttons_aligment = gtk_alignment_new(1, 1, 0, 0);
-  gtk_alignment_set_padding(GTK_ALIGNMENT(buttons_aligment), 20, 0, 0, 0);
-  gtk_container_add(GTK_CONTAINER(buttons_aligment), buttons_box);
-  gtk_container_add(GTK_CONTAINER(content_box), buttons_aligment);
-
+  gtk_container_add(GTK_CONTAINER(content_box), buttons_box);
   gtk_container_add(GTK_CONTAINER(self), main_box);
 
   return self;
@@ -368,7 +388,7 @@ GtkWidget* close_button_new()
   gtk_widget_set_can_focus(self, FALSE);
   gtk_widget_set_halign(self, GTK_ALIGN_START);
 
-  auto const& file = decoration::Style::Get()->ThemedFilePath(CLOSE_BUTTON_INACTIVE_FILE, {PKGDATADIR"/"});
+  auto const& file = decoration::Style::Get()->ThemedFilePath(CLOSE_BUTTON_INACTIVE_FILE, {PKGDATADIR});
   auto* img = gtk_image_new_from_file(file.c_str());
   gtk_container_add(GTK_CONTAINER(self), img);
   CLOSE_BUTTON(self)->priv->img = GTK_IMAGE(img);
@@ -402,13 +422,13 @@ static void close_button_class_init(CloseButtonClass* klass)
 
     auto new_flags = gtk_widget_get_state_flags(self);
     auto const& deco_style = decoration::Style::Get();
-    auto file = deco_style->ThemedFilePath(CLOSE_BUTTON_INACTIVE_FILE, {PKGDATADIR"/"});
+    auto file = deco_style->ThemedFilePath(CLOSE_BUTTON_INACTIVE_FILE, {PKGDATADIR});
 
     if (((new_flags & GTK_STATE_FLAG_PRELIGHT) && !gtk_widget_get_can_focus(self)) ||
         (new_flags & GTK_STATE_FLAG_FOCUSED))
     {
       auto const& basename = (new_flags & GTK_STATE_FLAG_ACTIVE) ? CLOSE_BUTTON_ACTIVE_FILE : CLOSE_BUTTON_FOCUSED_FILE;
-      file = deco_style->ThemedFilePath(basename, {PKGDATADIR"/"});
+      file = deco_style->ThemedFilePath(basename, {PKGDATADIR});
     }
 
     gtk_image_set_from_file(img, file.c_str());
@@ -442,8 +462,18 @@ struct ForceQuitDialog::Impl : sigc::trackable
 
   void UpdateWindowTime(Time time)
   {
-    gdk_x11_window_set_user_time(gtk_widget_get_window(dialog_), time);
+    auto gwindow = gtk_widget_get_window(dialog_);
+    gdk_x11_window_set_user_time(gwindow, time);
     gtk_widget_show_all(dialog_);
+
+    auto* dpy = gdk_x11_get_default_xdisplay();
+    auto xid = gdk_x11_window_get_xid(gwindow);
+    if (XWMHints *wmhints = XGetWMHints(dpy, xid))
+    {
+      wmhints->window_group = win_->id();
+      XSetWMHints(dpy, xid, wmhints);
+      XFree(wmhints);
+    }
   }
 
   void UpdateDialogPosition()
