@@ -301,7 +301,7 @@ void DBusProxy::Impl::OnPropertyChanged(GDBusProxy* proxy, GVariant* changed_pro
       }
     }
 
-    g_variant_iter_free (iter);
+    g_variant_iter_free(iter);
   }
 
   for (const gchar *property_name = *invalidated; property_name; property_name = *(++invalidated))
@@ -344,7 +344,8 @@ void DBusProxy::Impl::WaitForProxy(GCancellable* cancellable,
     // wait for the signal
     *con = proxy_acquired.connect([con, canc, timeout, callback] ()
     {
-      if (!g_cancellable_is_cancelled(canc)) callback(glib::Error());
+      if (!g_cancellable_is_cancelled(canc))
+        callback(glib::Error());
 
       timeout->Remove();
       con->disconnect();
@@ -390,12 +391,15 @@ void DBusProxy::Impl::Call(string const& method_name,
   if (!proxy_)
   {
     glib::Variant sinked_parameters(parameters);
-    glib::Object<GCancellable>canc(target_canc, glib::AddRef());
+    glib::Object<GCancellable> canc(target_canc, glib::AddRef());
+
     WaitForProxy(canc, timeout_msec, [this, method_name, sinked_parameters, callback, canc, flags, timeout_msec] (glib::Error const& err)
     {
       if (err)
       {
-        callback(glib::Variant(), err);
+        if (callback)
+          callback(glib::Variant(), err);
+
         LOG_WARNING(logger) << "Cannot call method " << method_name
                             << ": " << err;
       }
@@ -620,7 +624,7 @@ void DBusProxy::GetProperty(std::string const& name, ReplyCallback const& callba
                            [] (GObject *source, GAsyncResult *res, gpointer user_data) {
       glib::Error err;
       std::unique_ptr<ReplyCallback> callback(static_cast<ReplyCallback*>(user_data));
-      Variant result(g_dbus_connection_call_finish(G_DBUS_CONNECTION(source), res, &err));
+      Variant result(g_dbus_connection_call_finish(G_DBUS_CONNECTION(source), res, &err), StealRef());
 
       if (err)
       {
@@ -656,7 +660,7 @@ void DBusProxy::SetProperty(std::string const& name, GVariant* value)
                            nullptr, G_DBUS_CALL_FLAGS_NONE, -1, pimpl->cancellable_,
                            [] (GObject *source, GAsyncResult *res, gpointer user_data) {
       glib::Error err;
-      Variant result(g_dbus_connection_call_finish(G_DBUS_CONNECTION(source), res, &err));
+      Variant result(g_dbus_connection_call_finish(G_DBUS_CONNECTION(source), res, &err), StealRef());
       if (err)
       {
         LOG_ERROR(logger) << "Impossible to set property: " << err;
