@@ -127,7 +127,7 @@ GnomeManager::Impl::Impl(GnomeManager* manager, bool test_mode)
   }
 
   {
-    presence_proxy_ = std::make_shared<glib::DBusProxy>("org.gnome.SessionManager",
+    presence_proxy_ = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.gnome.SessionManager",
                                                         "/org/gnome/SessionManager/Presence",
                                                         "org.gnome.SessionManager.Presence");
 
@@ -161,6 +161,13 @@ GnomeManager::Impl::Impl(GnomeManager* manager, bool test_mode)
     });
   }
 
+  {
+    dm_seat_proxy_ = std::make_shared<glib::DBusProxy>("org.freedesktop.Accounts",
+                                                       ("/org/freedesktop/Accounts/User" + std::to_string(getuid())).c_str(),
+                                                       "org.freedesktop.Accounts.User",
+                                                       G_BUS_TYPE_SYSTEM);
+  }
+
   CallLogindMethod("CanHibernate", nullptr, [this] (GVariant* variant, glib::Error const& err) {
     if (err)
     {
@@ -435,6 +442,22 @@ void GnomeManager::Impl::CallConsoleKitMethod(std::string const& method, GVarian
   });
 }
 
+void GnomeManager::Impl::CallDisplayManagerSeatMethod(std::string const& method, GVariant* parameters)
+{
+  const char* xdg_seat_path = test_mode_ ? "/org/freedesktop/DisplayManager/Seat0" : g_getenv("XDG_SEAT_PATH");
+
+  auto proxy = std::make_shared<glib::DBusProxy>(test_mode_ ? testing::DBUS_NAME : "org.freedesktop.DisplayManager",
+                                                 glib::gchar_to_string(xdg_seat_path),
+                                                 "org.freedesktop.DisplayManager.Seat",
+                                                 test_mode_ ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM);
+  proxy->CallBegin(method, parameters, [this, proxy] (GVariant*, glib::Error const& e) {
+    if (e)
+    {
+      LOG_ERROR(logger) << "DisplayManager Seat call failed: " << e.Message();
+    }
+  });
+}
+
 void GnomeManager::Impl::LockScreen(bool prompt)
 {
   EnsureCancelPendingAction();
@@ -496,6 +519,13 @@ bool GnomeManager::Impl::HasInhibitors()
   return inhibitors.GetBool();
 }
 
+void GnomeManager::Impl::UserIconFile(std::function<void(std::string const&)> const& callback)
+{
+  dm_seat_proxy_->GetProperty("IconFile", [this, callback] (GVariant *value) {
+    callback(glib::Variant(value).GetString());
+  });
+}
+
 bool GnomeManager::Impl::IsUserInGroup(std::string const& user_name, std::string const& group_name)
 {
   auto group = getgrnam(group_name.c_str());
@@ -541,6 +571,11 @@ std::string GnomeManager::HostName() const
   return glib::gchar_to_string(g_get_host_name());
 }
 
+void GnomeManager::UserIconFile(std::function<void(std::string const&)> const& callback) const
+{
+  impl_->UserIconFile(callback);
+}
+
 void GnomeManager::ScreenSaverActivate()
 {
   screensaver_requested.emit(true);
@@ -553,12 +588,12 @@ void GnomeManager::ScreenSaverDeactivate()
 
 void GnomeManager::LockScreen()
 {
-  impl_->LockScreen(false);
+  impl_->LockScreen(/* prompt */ false);
 }
 
 void GnomeManager::PromptLockScreen()
 {
-  impl_->LockScreen(true);
+  impl_->LockScreen(/* prompt */ true);
 }
 
 void GnomeManager::Logout()
@@ -671,6 +706,11 @@ void GnomeManager::Hibernate()
   });
 }
 
+void GnomeManager::SwitchToGreeter()
+{
+  impl_->CallDisplayManagerSeatMethod("SwitchToGreeter");
+}
+
 bool GnomeManager::CanLock() const
 {
   if (is_locked())
