@@ -42,6 +42,7 @@
 #include "mock-application.h"
 #include "BamfApplicationManager.h"
 #include "bamf-mock-application.h"
+#include "unity-shared/UnitySettings.h"
 
 using namespace testmocks;
 using namespace unity::launcher;
@@ -61,7 +62,7 @@ const std::string DEVICES_URI = "unity://devices";
 namespace app
 {
  const std::string UBUNTU_ONE = BUILDDIR "/tests/data/applications/ubuntuone-installer.desktop";
- const std::string SW_CENTER = BUILDDIR "/tests/data/applications/ubuntu-software-center.desktop";
+ const std::string SW_CENTER = BUILDDIR "/tests/data/applications/org.gnome.Software.desktop";
  const std::string UPDATE_MANAGER = BUILDDIR "/tests/data/applications/update-manager.desktop";
  const std::string BZR_HANDLE_PATCH = BUILDDIR "/tests/data/applications/bzr-handle-patch.desktop";
  const std::string NO_ICON = BUILDDIR "/tests/data/applications/no-icon.desktop";
@@ -136,8 +137,10 @@ private:
 
 struct MockApplicationLauncherIcon : ApplicationLauncherIcon
 {
-  typedef NiceMock<MockApplicationLauncherIcon> Nice;
-  typedef nux::ObjectPtr<MockApplicationLauncherIcon::Nice> Ptr;
+  // NiceMock doesn't work well with Virtual Inheritance, so we need to disable it
+  //typedef NiceMock<MockApplicationLauncherIcon> Nice;
+  typedef MockApplicationLauncherIcon Nice;
+  typedef nux::ObjectPtr<MockApplicationLauncherIcon> Ptr;
   typedef bool Fake;
 
   MockApplicationLauncherIcon(Fake = true, std::string const& remote_uri = "")
@@ -154,7 +157,8 @@ struct MockApplicationLauncherIcon : ApplicationLauncherIcon
   }
 
   explicit MockApplicationLauncherIcon(ApplicationPtr const& app)
-    : ApplicationLauncherIcon(app)
+    : WindowedLauncherIcon(IconType::APPLICATION)
+    , ApplicationLauncherIcon(app)
   {
     ON_CALL(*this, Stick(_)).WillByDefault(Invoke([this] (bool save) { ApplicationLauncherIcon::Stick(save); }));
     ON_CALL(*this, UnStick()).WillByDefault(Invoke([this] { ApplicationLauncherIcon::UnStick(); }));
@@ -170,15 +174,18 @@ struct MockApplicationLauncherIcon : ApplicationLauncherIcon
   MOCK_CONST_METHOD0(GetRemoteUri, std::string());
   MOCK_METHOD1(Stick, void(bool));
   MOCK_METHOD0(UnStick, void());
-  MOCK_METHOD0(Quit, void());
+  MOCK_CONST_METHOD0(Quit, void());
 };
 
 struct MockVolumeLauncherIcon : public VolumeLauncherIcon
 {
   typedef nux::ObjectPtr<MockVolumeLauncherIcon> Ptr;
+  // typedef NiceMock<MockVolumeLauncherIcon> Nice;
+  typedef MockVolumeLauncherIcon Nice;
 
   MockVolumeLauncherIcon()
-    : VolumeLauncherIcon(Volume::Ptr(volume_ = new NiceMock<MockVolume>()),
+    : WindowedLauncherIcon(IconType::DEVICE)
+    , VolumeLauncherIcon(Volume::Ptr(volume_ = new NiceMock<MockVolume>()),
                          std::make_shared<MockDevicesSettings::Nice>(),
                          std::make_shared<MockDeviceNotificationDisplay::Nice>(),
                          std::make_shared<MockFileManager::Nice>())
@@ -269,7 +276,7 @@ protected:
     void DisconnectSignals()
     {
       ApplicationManager::Default().application_started.clear();
-      Impl()->device_section_.icon_added.clear();
+      Impl()->device_section_->icon_added.clear();
       Impl()->model_->icon_removed.clear();
       Impl()->model_->saved.clear();
       Impl()->model_->order_changed.clear();
@@ -303,7 +310,6 @@ TEST_F(TestLauncherController, Construction)
   EXPECT_FALSE(lc.Impl()->launcher_open);
   EXPECT_FALSE(lc.Impl()->launcher_keynav);
   EXPECT_FALSE(lc.Impl()->launcher_grabbed);
-  EXPECT_FALSE(lc.Impl()->reactivate_keynav);
   EXPECT_TRUE(lc.Impl()->keynav_restore_window_);
   EXPECT_EQ(lc.Impl()->launcher_key_press_time_, 0);
 
@@ -485,6 +491,25 @@ TEST_F(TestLauncherController, MonitorResizesLauncher)
   ASSERT_EQ(launcher_geo.height, monitor_geo.height - panel_style.PanelHeight());
 }
 
+TEST_F(TestLauncherController, LauncherPositionResetsOnGsettingsUpdated)
+{
+  glib::Object<GSettings> gsettings(g_settings_new("com.canonical.Unity.Launcher"));
+  g_settings_set_enum(gsettings, "launcher-position", static_cast<int>(LauncherPosition::LEFT));
+  nux::Geometry const& monitor_geo = uscreen.GetMonitorGeometry(0);
+  nux::Geometry launcher_geo = lc.launcher().GetAbsoluteGeometry();
+  ASSERT_EQ(launcher_geo.x, monitor_geo.x);
+  ASSERT_EQ(launcher_geo.y, monitor_geo.y + panel_style.PanelHeight(0));
+  ASSERT_EQ(launcher_geo.height, monitor_geo.height - panel_style.PanelHeight(0));
+
+  g_settings_set_enum(gsettings, "launcher-position", static_cast<int>(LauncherPosition::BOTTOM));
+  launcher_geo = lc.launcher().GetAbsoluteGeometry();
+  ASSERT_EQ(launcher_geo.x, monitor_geo.x);
+  ASSERT_EQ(launcher_geo.y, monitor_geo.y + monitor_geo.height - launcher_geo.height + 1);
+  ASSERT_EQ(launcher_geo.width, monitor_geo.width);
+
+  g_settings_reset(gsettings, "launcher-position");
+}
+
 TEST_F(TestLauncherController, IconCentersResetsOnMonitorsUpdated)
 {
   uscreen.SetupFakeMultiMonitor();
@@ -641,8 +666,8 @@ TEST_F(TestLauncherController, CreateFavoriteInvalidDesktopFile)
 
 TEST_F(TestLauncherController, CreateFavoriteDevice)
 {
-  lc.Impl()->device_section_ = MockDeviceLauncherSection();
-  auto const& icons = lc.Impl()->device_section_.GetIcons();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>();
+  auto const& icons = lc.Impl()->device_section_->GetIcons();
   auto const& device_icon = icons.front();
 
   ASSERT_TRUE(device_icon.IsValid());
@@ -904,8 +929,8 @@ TEST_F(TestLauncherController, AddDevices)
 {
   lc.ClearModel();
   lc.DisconnectSignals();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection();
-  auto const& icons = lc.Impl()->device_section_.GetIcons();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>();
+  auto const& icons = lc.Impl()->device_section_->GetIcons();
   auto const& device_icon1 = icons.front();
   auto const& device_icon2 = *(std::next(icons.begin()));
 
@@ -950,7 +975,7 @@ TEST_F(TestLauncherController, MigrateFavoritesUnneeded)
 TEST_F(TestLauncherController, SetupIcons)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>();
   lc.Impl()->expo_icon_->UnStick();
   lc.Impl()->desktop_icon_->UnStick();
   auto const& model = lc.Impl()->model_;
@@ -971,7 +996,7 @@ TEST_F(TestLauncherController, SetupIcons)
   fav = lc.Impl()->GetIconByUri(FavoriteStore::URI_PREFIX_APP + app::SW_CENTER);
   EXPECT_EQ(model->IconIndex(fav), ++icon_index);
 
-  for (auto const& device : lc.Impl()->device_section_.GetIcons())
+  for (auto const& device : lc.Impl()->device_section_->GetIcons())
     ASSERT_EQ(model->IconIndex(device), ++icon_index);
 
   fav = lc.Impl()->GetIconByUri(FavoriteStore::URI_PREFIX_APP + app::UPDATE_MANAGER);
@@ -996,7 +1021,7 @@ TEST_F(TestLauncherController, SetupIcons)
 TEST_F(TestLauncherController, ResetIconPriorities)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>();
   auto const& model = lc.Impl()->model_;
 
   favorite_store.AddFavorite(places::APPS_URI, -1);
@@ -1013,7 +1038,7 @@ TEST_F(TestLauncherController, ResetIconPriorities)
 
   int icon_index = -1;
 
-  for (auto const& device : lc.Impl()->device_section_.GetIcons())
+  for (auto const& device : lc.Impl()->device_section_->GetIcons())
     ASSERT_EQ(model->IconIndex(device), ++icon_index);
 
   auto fav = lc.Impl()->GetIconByUri(FavoriteStore::URI_PREFIX_APP + app::SW_CENTER);
@@ -1044,8 +1069,8 @@ TEST_F(TestLauncherController, ResetIconPriorities)
 TEST_F(TestLauncherController, GetLastIconPriorityUnSticky)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection(3);
-  auto const& device_icons = lc.Impl()->device_section_.GetIcons();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>(3);
+  auto const& device_icons = lc.Impl()->device_section_->GetIcons();
   auto const& last_device = device_icons.back();
 
   favorite_store.SetFavorites({ places::DEVICES_URI,
@@ -1061,8 +1086,8 @@ TEST_F(TestLauncherController, GetLastIconPriorityUnSticky)
 TEST_F(TestLauncherController, GetLastIconPriorityUnStickyWithAllStickyIcons)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection(3);
-  auto const& device_icons = lc.Impl()->device_section_.GetIcons();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>(3);
+  auto const& device_icons = lc.Impl()->device_section_->GetIcons();
   auto const& last_device = device_icons.back();
 
   favorite_store.SetFavorites({ places::DEVICES_URI,
@@ -1081,8 +1106,8 @@ TEST_F(TestLauncherController, GetLastIconPriorityUnStickyWithAllStickyIcons)
 TEST_F(TestLauncherController, GetLastIconPriorityUnStickyWithSomeStickyIcons)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection(3);
-  auto const& device_icons = lc.Impl()->device_section_.GetIcons();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>(3);
+  auto const& device_icons = lc.Impl()->device_section_->GetIcons();
   auto const& first_device = *(std::next(device_icons.rbegin()));
   auto const& last_device = device_icons.back();
 
@@ -1101,7 +1126,7 @@ TEST_F(TestLauncherController, GetLastIconPriorityUnStickyWithSomeStickyIcons)
 TEST_F(TestLauncherController, GetLastIconPriorityUnStickyWithNoIcons)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection(0);
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>(0);
   lc.Impl()->SetupIcons();
   lc.DisconnectSignals();
 
@@ -1112,7 +1137,7 @@ TEST_F(TestLauncherController, GetLastIconPriorityUnStickyWithNoIcons)
 TEST_F(TestLauncherController, GetLastIconPriorityUnStickyWithNoIconsAndUri)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection(0);
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>(0);
 
   favorite_store.SetFavorites({ places::DEVICES_URI,
                                 FavoriteStore::URI_PREFIX_APP + app::SW_CENTER });
@@ -1137,11 +1162,11 @@ TEST_F(TestLauncherController, GetLastIconPriorityUnStickyWithNoIconsAndUri)
 TEST_F(TestLauncherController, GetLastIconPrioritySticky)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection(3);
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>(3);
   lc.Impl()->SetupIcons();
   lc.DisconnectSignals();
 
-  auto const& device_icons = lc.Impl()->device_section_.GetIcons();
+  auto const& device_icons = lc.Impl()->device_section_->GetIcons();
   auto const& first_device = device_icons.front();
 
   int last_priority = lc.Impl()->GetLastIconPriority<VolumeLauncherIcon>("", true);
@@ -1151,8 +1176,8 @@ TEST_F(TestLauncherController, GetLastIconPrioritySticky)
 TEST_F(TestLauncherController, GetLastIconPriorityStickyWithAllStickyIcons)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection(3);
-  auto const& device_icons = lc.Impl()->device_section_.GetIcons();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>(3);
+  auto const& device_icons = lc.Impl()->device_section_->GetIcons();
   auto const& last_device = device_icons.back();
 
   favorite_store.SetFavorites({ places::DEVICES_URI,
@@ -1171,8 +1196,8 @@ TEST_F(TestLauncherController, GetLastIconPriorityStickyWithAllStickyIcons)
 TEST_F(TestLauncherController, GetLastIconPriorityStickyWithSomeStickyIcons)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection(3);
-  auto const& device_icons = lc.Impl()->device_section_.GetIcons();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>(3);
+  auto const& device_icons = lc.Impl()->device_section_->GetIcons();
   auto const& first_device = *(std::next(device_icons.rbegin()));
 
   favorite_store.SetFavorites({ places::DEVICES_URI,
@@ -1190,7 +1215,7 @@ TEST_F(TestLauncherController, GetLastIconPriorityStickyWithSomeStickyIcons)
 TEST_F(TestLauncherController, GetLastIconPriorityStickyWithNoIcons)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection(0);
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>(0);
   lc.Impl()->SetupIcons();
   lc.DisconnectSignals();
 
@@ -1241,8 +1266,8 @@ TEST_F(TestLauncherController, LauncherAddRequestApplicationStick)
 TEST_F(TestLauncherController, LauncherAddRequestDeviceAdd)
 {
   auto const& model = lc.Impl()->model_;
-  lc.Impl()->device_section_ = MockDeviceLauncherSection();
-  auto const& icons = lc.Impl()->device_section_.GetIcons();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>();
+  auto const& icons = lc.Impl()->device_section_->GetIcons();
   auto const& device_icon = icons.front();
   auto const& icon_uri = device_icon->RemoteUri();
 
@@ -1263,7 +1288,7 @@ TEST_F(TestLauncherController, LauncherAddRequestDeviceAdd)
 TEST_F(TestLauncherController, LauncherAddRequestDeviceStick)
 {
   auto const& model = lc.Impl()->model_;
-  MockVolumeLauncherIcon::Ptr device_icon(new NiceMock<MockVolumeLauncherIcon>());
+  MockVolumeLauncherIcon::Ptr device_icon(new MockVolumeLauncherIcon::Nice());
   lc.Impl()->RegisterIcon(device_icon, std::numeric_limits<int>::max());
 
   auto app_icons = model->GetSublist<ApplicationLauncherIcon>();
@@ -1288,7 +1313,7 @@ TEST_F(TestLauncherController, LauncherRemoveRequestApplicationUnStickAndQuit)
 
 TEST_F(TestLauncherController, LauncherRemoveRequestDeviceEjects)
 {
-  MockVolumeLauncherIcon::Ptr device_icon(new NiceMock<MockVolumeLauncherIcon>());
+  MockVolumeLauncherIcon::Ptr device_icon(new MockVolumeLauncherIcon::Nice());
 
   EXPECT_CALL(*(device_icon->volume_), CanBeEjected())
       .WillRepeatedly(Return(true));
@@ -1303,7 +1328,7 @@ TEST_F(TestLauncherController, LauncherRemoveRequestDeviceEjects)
 
 TEST_F(TestLauncherController, LauncherRemoveRequestDeviceStops)
 {
-  MockVolumeLauncherIcon::Ptr device_icon(new NiceMock<MockVolumeLauncherIcon>());
+  MockVolumeLauncherIcon::Ptr device_icon(new MockVolumeLauncherIcon::Nice());
 
   EXPECT_CALL(*(device_icon->volume_), CanBeEjected())
       .WillRepeatedly(Return(false));
@@ -1316,38 +1341,6 @@ TEST_F(TestLauncherController, LauncherRemoveRequestDeviceStops)
   lc.launcher().remove_request.emit(device_icon);
 }
 
-TEST_F(TestLauncherController, LauncherAddRequestSpecial)
-{
-  std::string desktop = app::BZR_HANDLE_PATCH;
-  std::string icon_uri = FavoriteStore::URI_PREFIX_APP + desktop;
-  ASSERT_FALSE(lc.Impl()->GetIconByUri(icon_uri).IsValid());
-  EXPECT_CALL(*unity_app_, LogEvent(_, _)).Times(0);
-
-  lc.Impl()->OnLauncherAddRequestSpecial(desktop, "", "", 0, 0, 32);
-
-  auto const& sw_center_icon = lc.Impl()->GetIconByUri(icon_uri);
-  ASSERT_TRUE(sw_center_icon.IsValid());
-  EXPECT_TRUE(sw_center_icon->IsSticky());
-  EXPECT_NE(dynamic_cast<SoftwareCenterLauncherIcon*>(sw_center_icon.GetPointer()), nullptr);
-}
-
-TEST_F(TestLauncherController, LauncherAddRequestSpecialIgnored)
-{
-  std::string desktop = app::BZR_HANDLE_PATCH;
-  std::string icon_uri = FavoriteStore::URI_PREFIX_APP + desktop;
-
-  MockApplicationLauncherIcon::Ptr bamf_icon(new MockApplicationLauncherIcon::Nice(desktop));
-  lc.Impl()->RegisterIcon(bamf_icon, std::numeric_limits<int>::max());
-  ASSERT_TRUE(lc.Impl()->GetIconByUri(icon_uri).IsValid());
-
-  EXPECT_CALL(*bamf_icon, Stick(false)).Times(0);
-
-  int previous_model_size = lc.Impl()->model_->Size();
-  lc.Impl()->OnLauncherAddRequestSpecial(desktop, "", "", 0, 0, 32);
-
-  EXPECT_EQ(previous_model_size, lc.Impl()->model_->Size());
-}
-
 TEST_F(TestLauncherController, SaveIconsOrder)
 {
   favorite_store.ClearFavorites();
@@ -1363,7 +1356,7 @@ TEST_F(TestLauncherController, SaveIconsOrder)
   invisible_app->SetQuirk(AbstractLauncherIcon::Quirk::VISIBLE, false);
   lc.Impl()->RegisterIcon(invisible_app, ++priority);
 
-  MockVolumeLauncherIcon::Ptr sticky_device(new NiceMock<MockVolumeLauncherIcon>());
+  MockVolumeLauncherIcon::Ptr sticky_device(new MockVolumeLauncherIcon::Nice());
   sticky_device->Stick(false);
   lc.Impl()->RegisterIcon(sticky_device, ++priority);
 
@@ -1394,7 +1387,7 @@ TEST_F(TestLauncherController, SaveIconsOrderWithOnlyStickyIcons)
   sticky_app->Stick(false);
   lc.Impl()->RegisterIcon(sticky_app, ++priority);
 
-  MockVolumeLauncherIcon::Ptr sticky_device(new NiceMock<MockVolumeLauncherIcon>());
+  MockVolumeLauncherIcon::Ptr sticky_device(new MockVolumeLauncherIcon::Nice());
   sticky_device->Stick(false);
   lc.Impl()->RegisterIcon(sticky_device, ++priority);
 
@@ -1423,7 +1416,7 @@ TEST_F(TestLauncherController, SaveIconsOrderTriesToKeepIconProvidersOrder)
   sticky_app->Stick(false);
   lc.Impl()->RegisterIcon(sticky_app, ++priority);
 
-  MockVolumeLauncherIcon::Ptr sticky_device(new NiceMock<MockVolumeLauncherIcon>());
+  MockVolumeLauncherIcon::Ptr sticky_device(new MockVolumeLauncherIcon::Nice());
   sticky_device->Stick(false);
   lc.Impl()->RegisterIcon(sticky_device, ++priority);
 
@@ -1448,7 +1441,7 @@ TEST_F(TestLauncherController, SaveIconsOrderTriesToKeepIconProvidersOrder2)
   sticky_app->Stick(false);
   lc.Impl()->RegisterIcon(sticky_app, ++priority);
 
-  MockVolumeLauncherIcon::Ptr sticky_device(new NiceMock<MockVolumeLauncherIcon>());
+  MockVolumeLauncherIcon::Ptr sticky_device(new MockVolumeLauncherIcon::Nice());
   sticky_device->Stick(false);
   lc.Impl()->RegisterIcon(sticky_device, ++priority);
 
@@ -1482,7 +1475,10 @@ TEST_F(TestLauncherController, SortAndUpdate)
 
   for (auto const& icon : *(lc.Impl()->model_))
   {
-    if (icon->IsVisible() && icon->GetIconType() == AbstractLauncherIcon::IconType::APPLICATION && expected_shortcut <= 10)
+    if (icon->IsVisible() &&
+        (icon->GetIconType() == AbstractLauncherIcon::IconType::APPLICATION ||
+         icon->GetIconType() == AbstractLauncherIcon::IconType::DEVICE) &&
+        expected_shortcut <= 10)
     {
       ASSERT_EQ(icon->GetShortcut(), std::to_string(expected_shortcut % 10)[0]);
       ++expected_shortcut;
@@ -1598,9 +1594,9 @@ TEST_F(TestLauncherController, OnFavoriteStoreFavoriteAddedStickAfter)
 TEST_F(TestLauncherController, OnFavoriteStoreFavoriteAddedDeviceSection)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>();
   auto const& model = lc.Impl()->model_;
-  auto const& icons = lc.Impl()->device_section_.GetIcons();
+  auto const& icons = lc.Impl()->device_section_->GetIcons();
   auto const& device_icon1(icons.front());
   auto const& device_icon2(*(std::next(icons.begin())));
 
@@ -1635,10 +1631,10 @@ TEST_F(TestLauncherController, OnFavoriteStoreFavoriteRemovedApplication)
 TEST_F(TestLauncherController, OnFavoriteStoreFavoriteRemovedDevice)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>();
   auto const& model = lc.Impl()->model_;
 
-  auto const& icons = lc.Impl()->device_section_.GetIcons();
+  auto const& icons = lc.Impl()->device_section_->GetIcons();
   auto const& device_icon(icons.front());
 
   favorite_store.SetFavorites({ lc.Impl()->expo_icon_->RemoteUri(),
@@ -1661,10 +1657,10 @@ TEST_F(TestLauncherController, OnFavoriteStoreFavoriteRemovedDevice)
 TEST_F(TestLauncherController, OnFavoriteStoreFavoriteRemovedDeviceSection)
 {
   lc.ClearModel();
-  lc.Impl()->device_section_ = MockDeviceLauncherSection();
+  lc.Impl()->device_section_ = std::make_shared<MockDeviceLauncherSection>();
   auto const& model = lc.Impl()->model_;
 
-  auto const& icons = lc.Impl()->device_section_.GetIcons();
+  auto const& icons = lc.Impl()->device_section_->GetIcons();
   auto const& device_icon1(icons.front());
   auto const& device_icon2(*(std::next(icons.begin())));
 
@@ -1959,10 +1955,9 @@ TEST_F(TestLauncherController, IconShowsOnQuickApplicationReopen)
   unity::glib::Object<BamfMockApplication> bamf_mock_application(bamf_mock_application_new());
   ApplicationPtr app(new unity::bamf::Application(mock_manager, unity::glib::object_cast<BamfApplication>(bamf_mock_application)));
 
-  MockApplicationLauncherIcon::Ptr our_icon;
-
+  AbstractLauncherIcon::Ptr our_icon;
   mock_manager.Default().application_started.emit(app);
-  
+
   app->title.changed.emit("Hello");
   auto app_icons = lc.Impl()->model_->GetSublist<ApplicationLauncherIcon>();
 
@@ -1972,7 +1967,7 @@ TEST_F(TestLauncherController, IconShowsOnQuickApplicationReopen)
     {
        our_icon = icon;
        break;
-    } 
+    }
   }
   ASSERT_TRUE(our_icon);
   EXPECT_FALSE(our_icon->removed);
