@@ -54,8 +54,8 @@
 #include "DashController.h"
 #include "UnitySettings.h"
 #include "DashStyle.h"
+#include "EdgeBarrierController.h"
 #include "FavoriteStoreGSettings.h"
-#include "FontSettings.h"
 #include "ShortcutController.h"
 #include "LauncherController.h"
 #include "LockScreenController.h"
@@ -139,7 +139,6 @@ public:
   void updateBlurDamage();
   void damageCutoff();
   void preparePaint (int ms);
-  void paintFboForOutput (CompOutput *output);
   void donePaint ();
 
   void RaiseInputWindows();
@@ -186,6 +185,8 @@ public:
 
   bool executeCommand(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool showDesktopKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
+  bool spreadAppWindowsInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
+  bool spreadAppWindowsAnywhereInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
   bool setKeyboardFocusKeyInitiate(CompAction* action, CompAction::State state, CompOption::Vector& options);
 
   bool altTabInitiateCommon(CompAction* action, switcher::ShowMode mode);
@@ -258,7 +259,10 @@ private:
     SHORTCUT_HINT
   };
 
-  void initAltTabNextWindow ();
+  static void InitNuxThread(nux::NThread* thread, void* data);
+  void InitUnityComponents();
+  bool InitPluginActions();
+  void InitAltTabNextWindow();
 
   void SendExecuteCommand();
 
@@ -266,16 +270,12 @@ private:
   void CreateSuperNewAction(char shortcut, impl::ActionModifiers flag);
   void EnableCancelAction(CancelActionTarget target, bool enabled, int modifiers = 0);
 
-  bool initPluginActions();
-  void initLauncher();
-
   void compizDamageNux(CompRegion const& region);
   void determineNuxDamage(CompRegion &nux_damage);
 
   void onRedrawRequested();
   void Relayout();
 
-  static void initUnity(nux::NThread* thread, void* InitData);
   static void OnStartKeyNav(GVariant* data, void* value);
   static void OnExitKeyNav(GVariant* data, void* value);
 
@@ -292,6 +292,7 @@ private:
   void OnInitiateSpread();
   void OnTerminateSpread();
 
+  void LoadPanelShadowTexture();
   void DamagePanelShadow();
 
   void OnViewHidden(nux::BaseWindow *bw);
@@ -319,6 +320,7 @@ private:
   void DamageBlurUpdateRegion(nux::Geometry const&);
 
   void ShowFirstRunHints();
+  void SpreadAppWindows(bool anywhere);
 
   std::unique_ptr<na::TickSource> tick_source_;
   std::unique_ptr<na::AnimationController> animation_controller_;
@@ -326,13 +328,13 @@ private:
   Settings unity_settings_;
   dash::Style    dash_style_;
   panel::Style   panel_style_;
-  FontSettings   font_settings_;
   internal::FavoriteStoreGSettings favorite_store_;
   ThumbnailGenerator thumbnail_generator_;
   lockscreen::Settings lockscreen_settings_;
 
   /* The window thread should be the last thing removed, as c++ does it in reverse order */
   std::unique_ptr<nux::WindowThread> wt;
+  WindowManager& WM;
 
   menu::Manager::Ptr menus_;
   std::shared_ptr<decoration::Manager> deco_manager_;
@@ -349,6 +351,7 @@ private:
   session::Controller::Ptr  session_controller_;
   lockscreen::DBusManager::Ptr screensaver_dbus_manager_;
   lockscreen::Controller::Ptr lockscreen_controller_;
+  ui::EdgeBarrierController::Ptr edge_barriers_;
   debug::DebugDBusInterface debugger_;
   std::unique_ptr<BGHash>   bghash_;
   spread::Filter::Ptr       spread_filter_;
@@ -368,7 +371,7 @@ private:
   typedef std::vector<CompActionPtr> ShortcutActions;
   ShortcutActions _shortcut_actions;
   std::map<CancelActionTarget, CompActionPtr> _escape_actions;
-  std::map<int, unsigned int> windows_for_monitor_;
+  std::unordered_map<int, unsigned int> windows_for_monitor_;
 
   /* keyboard-nav mode */
   CompWindow* newFocusedWindow;
